@@ -45,9 +45,9 @@
 #include <stdio.h> /* for printf */
 #include <stdarg.h> /* for va_list, va_start, etc. */
                                                
-#ifdef WIN32
-#define snprintf      _snprintf
-#define strcasecmp    stricmp
+#if defined(_MSC_VER)
+#define snprintf _snprintf
+#define strcasecmp stricmp
 #endif
 
 /* Private routines for icalproperty */
@@ -267,7 +267,7 @@ icalproperty_free (icalproperty* p)
 /*#define MAX_LINE_LEN 120*/
 
 static char*
-get_next_line_start (char *line_start, int chars_left)
+get_next_line_start (char *line_start, size_t chars_left)
 {
     char *pos;
 
@@ -320,9 +320,9 @@ get_next_line_start (char *line_start, int chars_left)
 static char*
 fold_property_line (char *text)
 {
-    size_t buf_size;
+    size_t buf_size, len, chars_left;
     char *buf, *buf_ptr, *line_start, *next_line_start;
-    int len, chars_left, first_line;
+    int first_line;
     char ch;
 
     /* Start with a buffer twice the size of our property line, so we almost
@@ -507,12 +507,15 @@ icalproperty_as_ical_string_r(icalproperty* prop)
 	char *str = icalvalue_as_ical_string_r(value);
 	if (str != 0)
 	    icalmemory_append_string(&buf, &buf_ptr, &buf_size, str);
+#if ICAL_ALLOW_EMPTY_PROPERTIES == 0
 	else
 	    icalmemory_append_string(&buf, &buf_ptr, &buf_size,"ERROR: No Value"); 
+#endif
 	free(str);
     } else {
+#if ICAL_ALLOW_EMPTY_PROPERTIES == 0
 	icalmemory_append_string(&buf, &buf_ptr, &buf_size,"ERROR: No Value"); 
-	
+#endif
     }
     
     icalmemory_append_string(&buf, &buf_ptr, &buf_size, newline);
@@ -807,14 +810,10 @@ void
 icalproperty_remove_parameter_by_ref(icalproperty* prop, icalparameter* parameter)
 {
     pvl_elem p;
-    icalparameter_kind kind;
-    const char *name;
 
     icalerror_check_arg_rv((prop!=0),"prop");
     icalerror_check_arg_rv((parameter!=0),"parameter");
 
-    kind = icalparameter_isa(parameter);
-
     for (p=pvl_head(prop->parameters);p != 0; p = pvl_next(p)) {
         icalparameter* p_param = (icalparameter *)pvl_data (p);
 
