@@ -23,6 +23,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include "icaltimezone.h"
 #include <string.h>
 
 #ifdef HAVE_STDINT_H
@@ -69,7 +70,7 @@
 #include <io.h>
 #endif
 
-#if defined(__APPLE__)
+#if defined(__APPLE__) || defined(__MINGW32__)
 #define bswap_16(x) (((x) << 8) & 0xff00) | (((x) >> 8 ) & 0xff)
 #define bswap_32 __builtin_bswap32
 #define bswap_64 __builtin_bswap64
@@ -93,8 +94,8 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <libical/icalerror.h>
-#include <icaltz-util.h>
+#include "icalerror.h"
+#include "icaltz-util.h"
 
 typedef struct 
 {
@@ -106,8 +107,6 @@ typedef struct
 	char	charcnt[4];			
 } tzinfo; 
 
-static int r_pos [] = {1, 2, 3, -2, -1};
-
 static char *search_paths [] = {"/usr/share/zoneinfo","/usr/lib/zoneinfo","/etc/zoneinfo","/usr/share/lib/zoneinfo"};
 static char *zdir = NULL;
 
@@ -193,9 +192,9 @@ set_zonedir (void)
 {
 	char file_path[PATH_MAX];
 	const char *fname = ZONES_TAB_SYSTEM_FILENAME;
-	int i;	
+	unsigned int i;
 
-	for (i = 0;i < NUM_SEARCH_PATHS; i++) {
+	for (i = 0; i < NUM_SEARCH_PATHS; i++) {
 		sprintf (file_path, "%s/%s", search_paths [i], fname);
 		if (!access (file_path, F_OK|R_OK)) {
 			zdir = search_paths [i];
@@ -204,7 +203,6 @@ set_zonedir (void)
 	}
 }
 
-
 const char *
 icaltzutil_get_zone_directory (void)
 {
@@ -214,40 +212,28 @@ icaltzutil_get_zone_directory (void)
 	return zdir;
 }
 
-/* Calculate the relative position of the week in a month from a date */
-static int
-calculate_pos (icaltimetype icaltime)
-{
-	int pos;
-
-	pos = (icaltime.day -1) / 7;
-
-	/* Check if pos 3 is the last occurence of the week day in the month */	
-	if (pos == 3 && ((icaltime.day + 7) > icaltime_days_in_month (icaltime.month, icaltime.year))) 
-		pos = 4;
-
-	return r_pos [pos];
-}
-
 icalcomponent*
 icaltzutil_fetch_timezone (const char *location)
 {
 	int ret = 0;
 	FILE *f;
 	tzinfo type_cnts;
-	unsigned int i, num_trans, num_types, num_chars, num_leaps, num_isstd, num_isgmt;
+	unsigned int i, num_trans, num_types = 0, num_chars, num_leaps, num_isstd, num_isgmt;
 	time_t *transitions = NULL;
-	time_t trans, start, end;
-	int *trans_idx = NULL, pos, sign, zidx, zp_idx, idx, prev_idx;
+	time_t start, end;
+	int *trans_idx = NULL, idx, prev_idx;
 	ttinfo *types = NULL;
 	char *znames = NULL, *full_path, *tzid, *r_trans, *temp;
 	leap *leaps = NULL;
 	icalcomponent *tz_comp = NULL, *comp = NULL;
 	icalproperty *icalprop;
-	icaltimetype dtstart, icaltime;
-	struct icalrecurrencetype ical_recur;
+	icaltimetype dtstart;
 	const char *basedir;
-	       
+
+	if(icaltimezone_get_builtin_tzdata()) {
+		return NULL;
+	}
+
 	basedir = icaltzutil_get_zone_directory();
 	if (!basedir) {
 		icalerror_set_errno (ICAL_FILE_ERROR);
@@ -256,7 +242,6 @@ icaltzutil_fetch_timezone (const char *location)
 
 	full_path = (char *) malloc (strlen (basedir) + strlen (location) + 2);
 	sprintf (full_path,"%s/%s",basedir, location);
-
 	if ((f = fopen (full_path, "rb")) == 0) {
 		icalerror_set_errno (ICAL_FILE_ERROR);
 		free (full_path);
@@ -343,14 +328,14 @@ icaltzutil_fetch_timezone (const char *location)
 
 	/* Read all the contents now */
 
-	for (i = 0; i < num_types; i++) 
+	for (i = 0; i < num_types; i++)
 		types [i].zname = zname_from_stridx (znames, types [i].abbr);
 
 	tz_comp = icalcomponent_new (ICAL_VTIMEZONE_COMPONENT);
 
 	/* Add tzid property */
 	tzid = (char *) malloc (strlen (ical_tzid_prefix) + strlen (location) + 8);
-	sprintf (tzid, "%sTzfile/%s", ical_tzid_prefix, location);
+	sprintf (tzid, "%s%s", ical_tzid_prefix, location);
 	icalprop = icalproperty_new_tzid (tzid);
 	icalcomponent_add_property (tz_comp, icalprop);
 	free (tzid);
@@ -416,7 +401,7 @@ error:
 	if (trans_idx)
 		free (trans_idx);
 	if (types) {
-		for (i = 0; i < num_types; i++) 
+		for (i = 0; i < num_types; i++)
 			if (types [i].zname)
 				free (types [i].zname);
 		free (types);
