@@ -55,10 +55,6 @@
 #define strcasecmp stricmp
 #endif
 
-#if _MAC_OS_
-#include "icalmemory_strdup.h"
-#endif
-
 #define TMP_BUF_SIZE 1024
 
 void print_datetime_to_string(char* str,  const struct icaltimetype *data);
@@ -319,20 +315,23 @@ static char* icalmemory_strdup_and_quote(const icalvalue* value,
             break;
 	    }
 
+/*issue74: \t is not escaped, but embedded literally.*/
 	    case '\t': {
-            icalmemory_append_string(&str,&str_p,&buf_sz,"\\t");
+            icalmemory_append_string(&str,&str_p,&buf_sz,"\t");
             break;
 	    }
+
+/*issue74: \r, \b and \f are not whitespace and are trashed.*/
 	    case '\r': {
-            icalmemory_append_string(&str,&str_p,&buf_sz,"\\r");
+            /*icalmemory_append_string(&str,&str_p,&buf_sz,"\\r");*/
             break;
 	    }
 	    case '\b': {
-            icalmemory_append_string(&str,&str_p,&buf_sz,"\\b");
+            /*icalmemory_append_string(&str,&str_p,&buf_sz,"\\b");*/
             break;
 	    }
 	    case '\f': {
-            icalmemory_append_string(&str,&str_p,&buf_sz,"\\f");
+            /*icalmemory_append_string(&str,&str_p,&buf_sz,"\\f");*/
             break;
 	    }
 
@@ -341,11 +340,15 @@ static char* icalmemory_strdup_and_quote(const icalvalue* value,
             /* unescaped COMMA is allowed in CATEGORIES property as its
                considered a list delimiter here, see:
                http://tools.ietf.org/html/rfc2445#section-4.3.11 */
-            if (icalproperty_isa(value->parent) == ICAL_CATEGORIES_PROPERTY) {
+            if ((icalproperty_isa(value->parent) == ICAL_CATEGORIES_PROPERTY) ||
+                (icalproperty_isa(value->parent) == ICAL_RESOURCES_PROPERTY) ||
+                (icalproperty_isa(value->parent) == ICAL_POLLPROPERTIES_PROPERTY)) {
                 icalmemory_append_char(&str,&str_p,&buf_sz,*p);
                 break;
             }
+/*issue74, we don't escape double quotes
 	    case '"':
+*/
 	    case '\\': {
             icalmemory_append_char(&str,&str_p,&buf_sz,'\\');
             icalmemory_append_char(&str,&str_p,&buf_sz,*p);
@@ -419,7 +422,7 @@ int simple_str_to_double(const char* from,
     }
 
     /*skip the white spaces at the beginning*/
-    while (cur && isspace (*cur))
+    while (cur && isspace ((int)*cur))
         cur++ ;
 
     start = cur ;
@@ -429,7 +432,7 @@ int simple_str_to_double(const char* from,
      * during the copy, we give ourselves a chance to convert the '.'
      * into the decimal separator of the current locale.
      */
-    while (cur && (isdigit (*cur) ||
+    while (cur && (isdigit ((int)*cur) ||
                    *cur == '.'    ||
                    *cur == '+'    ||
                    *cur == '-')){
@@ -553,6 +556,12 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
     case ICAL_CARLEVEL_VALUE:
         value = icalvalue_new_enum(kind, ICAL_CARLEVEL_X,str);
         break;
+    case ICAL_BUSYTYPE_VALUE:
+        value = icalvalue_new_enum(kind, ICAL_BUSYTYPE_X,str);
+        break;
+    case ICAL_POLLMODE_VALUE:
+        value = icalvalue_new_enum(kind, ICAL_POLLMODE_X,str);
+        break;
 
     case ICAL_INTEGER_VALUE:
 	    value = icalvalue_new_integer(atoi(str));
@@ -569,7 +578,7 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
                and reconstruct it as sections */
             t = strtol(str,0,10);
             /* add phantom seconds field */
-            if(abs(t)<9999){t *= 100; }
+            if(strlen(str)<7){t *= 100; }
             hours = (t/10000);
             minutes = (t-hours*10000)/100;
             seconds = (t-hours*10000-minutes*100);
@@ -610,7 +619,7 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
         }
   
         /*skip white spaces*/
-        while (cur && isspace (*cur)) {
+        while (cur && isspace ((int)*cur)) {
             ++cur ;
         }
 
@@ -624,7 +633,7 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
             goto geo_parsing_error ;
 
         /*skip white spaces*/
-        while (cur && isspace (*cur)) {
+        while (cur && isspace ((int)*cur)) {
             ++cur ;
         }
 
@@ -860,12 +869,9 @@ icalvalue_is_valid (const icalvalue* value)
 
 static char* icalvalue_binary_as_ical_string_r(const icalvalue* value) {
 
-    const char* data;
     char* str;
     icalerror_check_arg_rz( (value!=0),"value");
 
-    data = icalvalue_get_binary(value);
-
     str = (char*)icalmemory_new_buffer(60);
     snprintf(str, 60,"icalvalue_binary_as_ical_string is not implemented yet");
 
@@ -911,7 +917,7 @@ static char* icalvalue_utcoffset_as_ical_string_r(const icalvalue* value)
     m = (data - (h*3600))/ 60;
     s = (data - (h*3600) - (m*60));
 
-    if (s > 0)
+    if (s != 0)
     snprintf(str,9,"%c%02d%02d%02d",sign,abs(h),abs(m),abs(s));
     else
     snprintf(str,9,"%c%02d%02d",sign,abs(h),abs(m));
@@ -1240,6 +1246,8 @@ icalvalue_as_ical_string_r(const icalvalue* value)
     case ICAL_STATUS_VALUE:
     case ICAL_TRANSP_VALUE:
     case ICAL_CLASS_VALUE:
+    case ICAL_BUSYTYPE_VALUE:
+    case ICAL_POLLMODE_VALUE:
         if(value->x_value !=0){
             return icalmemory_strdup(value->x_value);
         }
