@@ -42,7 +42,10 @@
 #include <stdlib.h> /* For rand */
 #include <string.h> /* for strrchr, strdup*/
 #if defined(HAVE_UNISTD_H)
-#include <unistd.h>   /* for getopt */
+#include <unistd.h>   /* for getopt, sleep */
+#endif
+#ifdef WIN32
+#include <windows.h> /* for Sleep */
 #endif
 
 /*int sspm_parse_mime(struct sspm_part *parts, 
@@ -96,8 +99,10 @@ int main(int argc, char* argv[]) {
 
     FILE *f;
     int c;
+#if !defined(HAVE_UNISTD_H)
     extern char *optarg;
     extern int optind, optopt;
+#endif
     int errflg=0;
     char* program_name;
 
@@ -324,7 +329,7 @@ int main(int argc, char* argv[]) {
 
     if (opt.sleep != 0){
 #ifdef WIN32
-        _sleep(opt.sleep*1000);
+        Sleep(opt.sleep*1000);
 #else
 	sleep(opt.sleep);
 #endif
